@@ -1,5 +1,5 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
 #include "cmGhsMultiTargetGenerator.h"
 
 #include <algorithm>
@@ -15,6 +15,7 @@
 #include "cmCustomCommand.h"
 #include "cmCustomCommandGenerator.h"
 #include "cmGeneratedFileStream.h"
+#include "cmGeneratorOptions.h"
 #include "cmGeneratorTarget.h"
 #include "cmGlobalGhsMultiGenerator.h"
 #include "cmLinkLineComputer.h" // IWYU pragma: keep
@@ -62,7 +63,7 @@ void cmGhsMultiTargetGenerator::Generate()
       this->TargetNameReal =
         this->GeneratorTarget->GetExecutableNames(this->ConfigName).Real;
       if (this->cmGhsMultiTargetGenerator::DetermineIfIntegrityApp()) {
-        this->TagType = GhsMultiGpj::INTERGRITY_APPLICATION;
+        this->TagType = GhsMultiGpj::INTEGRITY_APPLICATION;
       } else {
         this->TagType = GhsMultiGpj::PROGRAM;
       }
@@ -147,7 +148,7 @@ void cmGhsMultiTargetGenerator::GenerateTarget()
   GhsMultiGpj::WriteGpjTag(this->TagType, fout);
 
   if (this->TagType != GhsMultiGpj::CUSTOM_TARGET) {
-    const std::string language(
+    std::string const language(
       this->GeneratorTarget->GetLinkerLanguage(this->ConfigName));
     this->WriteTargetSpecifics(fout, this->ConfigName);
     this->SetCompilerFlags(this->ConfigName, language);
@@ -169,7 +170,7 @@ cmGlobalGhsMultiGenerator* cmGhsMultiTargetGenerator::GetGlobalGenerator()
 }
 
 void cmGhsMultiTargetGenerator::WriteTargetSpecifics(std::ostream& fout,
-                                                     const std::string& config)
+                                                     std::string const& config)
 {
   std::string outpath;
 
@@ -195,14 +196,14 @@ void cmGhsMultiTargetGenerator::WriteTargetSpecifics(std::ostream& fout,
 }
 
 void cmGhsMultiTargetGenerator::SetCompilerFlags(std::string const& config,
-                                                 const std::string& language)
+                                                 std::string const& language)
 {
   auto i = this->FlagsByLanguage.find(language);
   if (i == this->FlagsByLanguage.end()) {
     std::string flags;
     this->LocalGenerator->AddLanguageFlags(
       flags, this->GeneratorTarget, cmBuildStep::Compile, language, config);
-    this->LocalGenerator->AddCMP0018Flags(flags, this->GeneratorTarget,
+    this->LocalGenerator->AddFeatureFlags(flags, this->GeneratorTarget,
                                           language, config);
     this->LocalGenerator->AddVisibilityPresetFlags(
       flags, this->GeneratorTarget, language);
@@ -223,7 +224,7 @@ void cmGhsMultiTargetGenerator::SetCompilerFlags(std::string const& config,
   }
 }
 
-std::string cmGhsMultiTargetGenerator::GetDefines(const std::string& language,
+std::string cmGhsMultiTargetGenerator::GetDefines(std::string const& language,
                                                   std::string const& config)
 {
   auto i = this->DefinesByLanguage.find(language);
@@ -245,14 +246,14 @@ std::string cmGhsMultiTargetGenerator::GetDefines(const std::string& language,
 
 void cmGhsMultiTargetGenerator::WriteCompilerFlags(std::ostream& fout,
                                                    std::string const&,
-                                                   const std::string& language)
+                                                   std::string const& language)
 {
   auto flagsByLangI = this->FlagsByLanguage.find(language);
   if (flagsByLangI != this->FlagsByLanguage.end()) {
     if (!flagsByLangI->second.empty()) {
       std::vector<std::string> ghsCompFlags =
         cmSystemTools::ParseArguments(flagsByLangI->second);
-      for (const std::string& f : ghsCompFlags) {
+      for (std::string const& f : ghsCompFlags) {
         fout << "    " << f << '\n';
       }
     }
@@ -260,7 +261,7 @@ void cmGhsMultiTargetGenerator::WriteCompilerFlags(std::ostream& fout,
 }
 
 void cmGhsMultiTargetGenerator::WriteCompilerDefinitions(
-  std::ostream& fout, const std::string& config, const std::string& language)
+  std::ostream& fout, std::string const& config, std::string const& language)
 {
   std::vector<std::string> compileDefinitions;
   this->GeneratorTarget->GetCompileDefinitions(compileDefinitions, config,
@@ -271,8 +272,8 @@ void cmGhsMultiTargetGenerator::WriteCompilerDefinitions(
 }
 
 void cmGhsMultiTargetGenerator::WriteIncludes(std::ostream& fout,
-                                              const std::string& config,
-                                              const std::string& language)
+                                              std::string const& config,
+                                              std::string const& language)
 {
   std::vector<std::string> includes;
   this->LocalGenerator->GetIncludeDirectories(includes, this->GeneratorTarget,
@@ -286,7 +287,7 @@ void cmGhsMultiTargetGenerator::WriteIncludes(std::ostream& fout,
 void cmGhsMultiTargetGenerator::WriteTargetLinkLine(std::ostream& fout,
                                                     std::string const& config)
 {
-  if (this->TagType == GhsMultiGpj::INTERGRITY_APPLICATION) {
+  if (this->TagType == GhsMultiGpj::INTEGRITY_APPLICATION) {
     return;
   }
 
@@ -307,14 +308,14 @@ void cmGhsMultiTargetGenerator::WriteTargetLinkLine(std::ostream& fout,
 
   // write out link options
   std::vector<std::string> lopts = cmSystemTools::ParseArguments(linkFlags);
-  for (const std::string& l : lopts) {
+  for (std::string const& l : lopts) {
     fout << "    " << l << '\n';
   }
 
   // write out link search paths
   // must be quoted for paths that contain spaces
   std::vector<std::string> lpath = cmSystemTools::ParseArguments(linkPath);
-  for (const std::string& l : lpath) {
+  for (std::string const& l : lpath) {
     fout << "    -L\"" << l << "\"\n";
   }
 
@@ -324,7 +325,7 @@ void cmGhsMultiTargetGenerator::WriteTargetLinkLine(std::ostream& fout,
 
   std::vector<std::string> llibs =
     cmSystemTools::ParseArguments(linkLibraries);
-  for (const std::string& l : llibs) {
+  for (std::string const& l : llibs) {
     if (l.compare(0, 2, "-l") == 0) {
       fout << "    \"" << l << "\"\n";
     } else {
@@ -370,7 +371,7 @@ void cmGhsMultiTargetGenerator::WriteBuildEvents(std::ostream& fout)
 }
 
 void cmGhsMultiTargetGenerator::WriteBuildEventsHelper(
-  std::ostream& fout, const std::vector<cmCustomCommand>& ccv,
+  std::ostream& fout, std::vector<cmCustomCommand> const& ccv,
   std::string const& name, std::string const& cmd)
 {
   int cmdcount = 0;
@@ -399,7 +400,7 @@ void cmGhsMultiTargetGenerator::WriteBuildEventsHelper(
     } else {
       fout << fname << "\n    :outputName=\"" << fname << ".rule\"\n";
     }
-    for (const auto& byp : ccg.GetByproducts()) {
+    for (auto const& byp : ccg.GetByproducts()) {
       fout << "    :extraOutputFile=\"" << byp << "\"\n";
     }
   }
@@ -496,13 +497,13 @@ void cmGhsMultiTargetGenerator::WriteCustomCommandsHelper(
 }
 
 void cmGhsMultiTargetGenerator::WriteSourceProperty(
-  std::ostream& fout, const cmSourceFile* sf, std::string const& propName,
+  std::ostream& fout, cmSourceFile const* sf, std::string const& propName,
   std::string const& propFlag)
 {
   cmValue prop = sf->GetProperty(propName);
   if (prop) {
     cmList list{ *prop };
-    for (const std::string& p : list) {
+    for (std::string const& p : list) {
       fout << "    " << propFlag << p << '\n';
     }
   }
@@ -531,7 +532,7 @@ void cmGhsMultiTargetGenerator::WriteSources(std::ostream& fout_proj)
   }
 
   /* list of known groups and the order they are displayed in a project file */
-  const std::vector<std::string> standardGroups = {
+  std::vector<std::string> const standardGroups = {
     "CMake Rules",  "Header Files",     "Source Files",
     "Object Files", "Object Libraries", "Resources"
   };
@@ -544,7 +545,7 @@ void cmGhsMultiTargetGenerator::WriteSources(std::ostream& fout_proj)
    *   in the order used by std::map.
    */
   int i = 0;
-  for (const std::string& gn : standardGroups) {
+  for (std::string const& gn : standardGroups) {
     auto n = groupNames.find(gn);
     if (n != groupNames.end()) {
       groupFilesList[i] = *n;
@@ -570,7 +571,7 @@ void cmGhsMultiTargetGenerator::WriteSources(std::ostream& fout_proj)
     }
   }
 
-  for (const auto& n : groupNames) {
+  for (auto const& n : groupNames) {
     groupFilesList[i] = n;
     i += 1;
   }
@@ -593,7 +594,7 @@ void cmGhsMultiTargetGenerator::WriteSources(std::ostream& fout_proj)
   for (auto& sg : groupFilesList) {
     std::ostream* fout;
     bool useProjectFile =
-      cmIsOn(this->GeneratorTarget->GetProperty("GHS_NO_SOURCE_GROUP_FILE")) ||
+      this->GeneratorTarget->GetProperty("GHS_NO_SOURCE_GROUP_FILE").IsOn() ||
       this->Makefile->IsOn("CMAKE_GHS_NO_SOURCE_GROUP_FILE");
     if (useProjectFile || sg.empty()) {
       fout = &fout_proj;
@@ -629,7 +630,7 @@ void cmGhsMultiTargetGenerator::WriteSources(std::ostream& fout_proj)
 
     if (sg != "CMake Rules") {
       /* output rule for each source file */
-      for (const cmSourceFile* si : groupFiles[sg]) {
+      for (cmSourceFile const* si : groupFiles[sg]) {
         bool compile = true;
         // Convert filename to native system
         // WORKAROUND: GHS MULTI 6.1.4 and 6.1.6 are known to need backslash on
@@ -693,7 +694,7 @@ void cmGhsMultiTargetGenerator::WriteSources(std::ostream& fout_proj)
         std::string fext = ".sh";
 #endif
         for (auto& sf : customCommands) {
-          const cmCustomCommand* cc = sf->GetCustomCommand();
+          cmCustomCommand const* cc = sf->GetCustomCommand();
           cmCustomCommandGenerator ccg(*cc, this->ConfigName,
                                        this->LocalGenerator);
 
@@ -734,14 +735,14 @@ void cmGhsMultiTargetGenerator::WriteCustomCommandLine(
    * the outputs is manually deleted.
    */
   bool specifyExtra = true;
-  for (const auto& out : ccg.GetOutputs()) {
+  for (auto const& out : ccg.GetOutputs()) {
     fout << fname << '\n';
     fout << "    :outputName=\"" << out << "\"\n";
     if (specifyExtra) {
-      for (const auto& byp : ccg.GetByproducts()) {
+      for (auto const& byp : ccg.GetByproducts()) {
         fout << "    :extraOutputFile=\"" << byp << "\"\n";
       }
-      for (const auto& dep : ccg.GetDepends()) {
+      for (auto const& dep : ccg.GetDepends()) {
         fout << "    :depends=\"" << dep << "\"\n";
       }
       specifyExtra = false;
@@ -750,7 +751,7 @@ void cmGhsMultiTargetGenerator::WriteCustomCommandLine(
 }
 
 std::string cmGhsMultiTargetGenerator::WriteObjectLangOverride(
-  const cmSourceFile* sourceFile)
+  cmSourceFile const* sourceFile)
 {
   std::string ret;
   cmValue rawLangProp = sourceFile->GetProperty("LANGUAGE");
@@ -764,7 +765,7 @@ std::string cmGhsMultiTargetGenerator::WriteObjectLangOverride(
 bool cmGhsMultiTargetGenerator::DetermineIfIntegrityApp()
 {
   if (cmValue p = this->GeneratorTarget->GetProperty("ghs_integrity_app")) {
-    return cmIsOn(*p);
+    return p.IsOn();
   }
   std::vector<cmSourceFile*> sources;
   this->GeneratorTarget->GetSourceFiles(sources, this->ConfigName);
@@ -801,7 +802,7 @@ bool cmGhsMultiTargetGenerator::VisitCustomCommand(
   if (perm.find(si) == perm.end()) {
     /* set temporary mark; check if revisit*/
     if (temp.insert(si).second) {
-      for (const auto& di : si->GetCustomCommand()->GetDepends()) {
+      for (auto const& di : si->GetCustomCommand()->GetDepends()) {
         cmSourceFile const* sf =
           this->GeneratorTarget->GetLocalGenerator()->GetSourceFileWithOutput(
             di);
