@@ -1,5 +1,5 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
 #include "cmMakefileTargetGenerator.h"
 
 #include <algorithm>
@@ -25,6 +25,7 @@
 #include "cmFileSet.h"
 #include "cmGeneratedFileStream.h"
 #include "cmGeneratorExpression.h"
+#include "cmGeneratorOptions.h"
 #include "cmGeneratorTarget.h"
 #include "cmGlobalUnixMakefileGenerator3.h"
 #include "cmLinkLineComputer.h" // IWYU pragma: keep
@@ -65,7 +66,7 @@ cmMakefileTargetGenerator::cmMakefileTargetGenerator(cmGeneratorTarget* target)
   this->NoRuleMessages = false;
   if (cmValue ruleStatus =
         cm->GetState()->GetGlobalProperty("RULE_MESSAGES")) {
-    this->NoRuleMessages = cmIsOff(*ruleStatus);
+    this->NoRuleMessages = ruleStatus.IsOff();
   }
   switch (this->GeneratorTarget->GetPolicyStatusCMP0113()) {
     case cmPolicies::WARN:
@@ -74,8 +75,6 @@ cmMakefileTargetGenerator::cmMakefileTargetGenerator(cmGeneratorTarget* target)
       this->CMP0113New = false;
       break;
     case cmPolicies::NEW:
-    case cmPolicies::REQUIRED_IF_USED:
-    case cmPolicies::REQUIRED_ALWAYS:
       this->CMP0113New = true;
       break;
   }
@@ -111,7 +110,7 @@ std::unique_ptr<cmMakefileTargetGenerator> cmMakefileTargetGenerator::New(
   return result;
 }
 
-std::string cmMakefileTargetGenerator::GetConfigName() const
+std::string const& cmMakefileTargetGenerator::GetConfigName() const
 {
   auto const& configNames = this->LocalGenerator->GetConfigNames();
   assert(configNames.size() == 1);
@@ -119,7 +118,7 @@ std::string cmMakefileTargetGenerator::GetConfigName() const
 }
 
 void cmMakefileTargetGenerator::GetDeviceLinkFlags(
-  std::string& linkFlags, const std::string& linkLanguage)
+  std::string& linkFlags, std::string const& linkLanguage)
 {
   cmGeneratorTarget::DeviceLinkSetter setter(*this->GetGeneratorTarget());
 
@@ -134,7 +133,7 @@ void cmMakefileTargetGenerator::GetDeviceLinkFlags(
 }
 
 void cmMakefileTargetGenerator::GetTargetLinkFlags(
-  std::string& flags, const std::string& linkLanguage)
+  std::string& flags, std::string const& linkLanguage)
 {
   this->LocalGenerator->AppendFlags(
     flags, this->GeneratorTarget->GetSafeProperty("LINK_FLAGS"));
@@ -157,6 +156,8 @@ void cmMakefileTargetGenerator::GetTargetLinkFlags(
     flags, this->GeneratorTarget, this->GetConfigName(), linkLanguage);
   this->LocalGenerator->AppendPositionIndependentLinkerFlags(
     flags, this->GeneratorTarget, this->GetConfigName(), linkLanguage);
+  this->LocalGenerator->AppendWarningAsErrorLinkerFlags(
+    flags, this->GeneratorTarget, linkLanguage);
   this->LocalGenerator->AppendDependencyInfoLinkerFlags(
     flags, this->GeneratorTarget, this->GetConfigName(), linkLanguage);
 }
@@ -209,7 +210,7 @@ void cmMakefileTargetGenerator::WriteTargetBuildRules()
   // -- Write the custom commands for this target
 
   // Evaluates generator expressions and expands prop_value
-  auto evaluatedFiles = [this](const std::string& prop_value) -> cmList {
+  auto evaluatedFiles = [this](std::string const& prop_value) -> cmList {
     cmList files{ cmGeneratorExpression::Evaluate(
       prop_value, this->LocalGenerator, this->GetConfigName(),
       this->GeneratorTarget) };
@@ -244,13 +245,15 @@ void cmMakefileTargetGenerator::WriteTargetBuildRules()
   }
 
   // add custom commands to the clean rules?
-  bool const clean = cmIsOff(this->Makefile->GetProperty("CLEAN_NO_CUSTOM"));
+  bool const clean = this->Makefile->GetProperty("CLEAN_NO_CUSTOM").IsOff();
 
   // First generate the object rule files.  Save a list of all object
   // files for this target.
   std::vector<cmSourceFile const*> customCommands;
   this->GeneratorTarget->GetCustomCommands(customCommands,
                                            this->GetConfigName());
+  std::vector<std::string> codegen_depends;
+  codegen_depends.reserve(customCommands.size());
   for (cmSourceFile const* sf : customCommands) {
     if (this->CMP0113New &&
         !this->LocalGenerator->GetCommandsVisited(this->GeneratorTarget)
@@ -262,17 +265,44 @@ void cmMakefileTargetGenerator::WriteTargetBuildRules()
                                  this->GetConfigName(), this->LocalGenerator);
     this->GenerateCustomRuleFile(ccg);
     if (clean) {
-      const std::vector<std::string>& outputs = ccg.GetOutputs();
+      std::vector<std::string> const& outputs = ccg.GetOutputs();
       for (std::string const& output : outputs) {
         this->CleanFiles.insert(
           this->LocalGenerator->MaybeRelativeToCurBinDir(output));
       }
-      const std::vector<std::string>& byproducts = ccg.GetByproducts();
+      std::vector<std::string> const& byproducts = ccg.GetByproducts();
       for (std::string const& byproduct : byproducts) {
         this->CleanFiles.insert(
           this->LocalGenerator->MaybeRelativeToCurBinDir(byproduct));
       }
     }
+
+    if (ccg.GetCC().GetCodegen()) {
+      std::string const& output = ccg.GetOutputs().front();
+
+      // We always attach the actual commands to the first output.
+      codegen_depends.emplace_back(output);
+    }
+  }
+
+  // Some make tools need a special dependency for an empty rule.
+  if (codegen_depends.empty()) {
+    std::string hack = this->GlobalGenerator->GetEmptyRuleHackDepends();
+    if (!hack.empty()) {
+      codegen_depends.emplace_back(std::move(hack));
+    }
+  }
+
+  // Construct the codegen target.
+  {
+    std::string const codegenTarget = cmStrCat(
+      this->LocalGenerator->GetRelativeTargetDirectory(this->GeneratorTarget),
+      "/codegen");
+
+    // Write the rule.
+    this->LocalGenerator->WriteMakeRule(*this->BuildFileStream, nullptr,
+                                        codegenTarget, codegen_depends, {},
+                                        true);
   }
 
   // Add byproducts from build events to the clean rules
@@ -285,10 +315,10 @@ void cmMakefileTargetGenerator::WriteTargetBuildRules()
     cm::append(buildEventCommands,
                this->GeneratorTarget->GetPostBuildCommands());
 
-    for (const auto& be : buildEventCommands) {
+    for (auto const& be : buildEventCommands) {
       cmCustomCommandGenerator beg(be, this->GetConfigName(),
                                    this->LocalGenerator);
-      const std::vector<std::string>& byproducts = beg.GetByproducts();
+      std::vector<std::string> const& byproducts = beg.GetByproducts();
       for (std::string const& byproduct : byproducts) {
         this->CleanFiles.insert(
           this->LocalGenerator->MaybeRelativeToCurBinDir(byproduct));
@@ -383,7 +413,7 @@ void cmMakefileTargetGenerator::WriteTargetBuildRules()
 
 void cmMakefileTargetGenerator::WriteCommonCodeRules()
 {
-  const char* root = (this->Makefile->IsOn("CMAKE_MAKE_INCLUDE_FROM_ROOT")
+  char const* root = (this->Makefile->IsOn("CMAKE_MAKE_INCLUDE_FROM_ROOT")
                         ? "$(CMAKE_BINARY_DIR)/"
                         : "");
 
@@ -545,7 +575,7 @@ void cmMakefileTargetGenerator::WriteTargetLanguageFlags()
       this->GeneratorTarget->GetAppleArchs(this->GetConfigName(), language);
     architectures.emplace_back();
 
-    for (const std::string& arch : architectures) {
+    for (std::string const& arch : architectures) {
       std::string flags =
         this->GetFlags(language, this->GetConfigName(), arch);
       if (escapeOctothorpe) {
@@ -558,7 +588,7 @@ void cmMakefileTargetGenerator::WriteTargetLanguageFlags()
 }
 
 void cmMakefileTargetGenerator::MacOSXContentGeneratorType::operator()(
-  cmSourceFile const& source, const char* pkgloc, const std::string& config)
+  cmSourceFile const& source, char const* pkgloc, std::string const& config)
 {
   // Skip OS X content when not building a Framework or Bundle.
   if (!this->Generator->GetGeneratorTarget()->IsBundleOnApple()) {
@@ -604,7 +634,7 @@ void cmMakefileTargetGenerator::WriteObjectRuleFiles(
   cmSourceFile const& source)
 {
   // Identify the language of the source file.
-  const std::string& lang = source.GetLanguage();
+  std::string const& lang = source.GetLanguage();
   if (lang.empty()) {
     // don't know anything about this file so skip it
     return;
@@ -613,8 +643,9 @@ void cmMakefileTargetGenerator::WriteObjectRuleFiles(
   // Use compiler to generate dependencies, if supported.
   bool const compilerGenerateDeps =
     this->GlobalGenerator->SupportsCompilerDependencies() &&
-    cmIsOn(this->Makefile->GetDefinition(
-      cmStrCat("CMAKE_", lang, "_DEPENDS_USE_COMPILER")));
+    this->Makefile
+      ->GetDefinition(cmStrCat("CMAKE_", lang, "_DEPENDS_USE_COMPILER"))
+      .IsOn();
   auto const scanner = compilerGenerateDeps ? cmDependencyScannerKind::Compiler
                                             : cmDependencyScannerKind::CMake;
 
@@ -653,9 +684,7 @@ void cmMakefileTargetGenerator::WriteObjectRuleFiles(
   // The object file should be checked for dependency integrity.
   std::string objFullPath =
     cmStrCat(this->LocalGenerator->GetCurrentBinaryDirectory(), '/', obj);
-  objFullPath = cmSystemTools::CollapseFullPath(objFullPath);
-  std::string const srcFullPath =
-    cmSystemTools::CollapseFullPath(source.GetFullPath());
+  std::string const srcFullPath = source.GetFullPath();
   this->LocalGenerator->AddImplicitDepends(this->GeneratorTarget, lang,
                                            objFullPath, srcFullPath, scanner);
 
@@ -671,16 +700,13 @@ void cmMakefileTargetGenerator::WriteObjectRuleFiles(
   std::string const configUpper = cmSystemTools::UpperCase(config);
 
   // Add precompile headers dependencies
-  std::vector<std::string> architectures =
-    this->GeneratorTarget->GetAppleArchs(config, lang);
-  if (architectures.empty()) {
-    architectures.emplace_back();
-  }
+  std::vector<std::string> pchArchs =
+    this->GeneratorTarget->GetPchArchs(config, lang);
 
   std::string filterArch;
   std::unordered_map<std::string, std::string> pchSources;
-  for (const std::string& arch : architectures) {
-    const std::string pchSource =
+  for (std::string const& arch : pchArchs) {
+    std::string const pchSource =
       this->GeneratorTarget->GetPchSource(config, lang, arch);
     if (pchSource == source.GetFullPath()) {
       filterArch = arch;
@@ -691,7 +717,7 @@ void cmMakefileTargetGenerator::WriteObjectRuleFiles(
   }
 
   if (!pchSources.empty() && !source.GetProperty("SKIP_PRECOMPILE_HEADERS")) {
-    for (const std::string& arch : architectures) {
+    for (std::string const& arch : pchArchs) {
       std::string const& pchHeader =
         this->GeneratorTarget->GetPchHeader(config, lang, arch);
       depends.push_back(pchHeader);
@@ -760,9 +786,9 @@ void cmMakefileTargetGenerator::WriteObjectRuleFiles(
   }
 
   // Add flags from source file properties.
-  const std::string COMPILE_FLAGS("COMPILE_FLAGS");
+  std::string const COMPILE_FLAGS("COMPILE_FLAGS");
   if (cmValue cflags = source.GetProperty(COMPILE_FLAGS)) {
-    const std::string& evaluatedFlags =
+    std::string const& evaluatedFlags =
       genexInterpreter.Evaluate(*cflags, COMPILE_FLAGS);
     this->LocalGenerator->AppendFlags(flags, evaluatedFlags);
     *this->FlagFileStream << "# Custom flags: " << relativeObj
@@ -770,9 +796,9 @@ void cmMakefileTargetGenerator::WriteObjectRuleFiles(
                           << "\n";
   }
 
-  const std::string COMPILE_OPTIONS("COMPILE_OPTIONS");
+  std::string const COMPILE_OPTIONS("COMPILE_OPTIONS");
   if (cmValue coptions = source.GetProperty(COMPILE_OPTIONS)) {
-    const std::string& evaluatedOptions =
+    std::string const& evaluatedOptions =
       genexInterpreter.Evaluate(*coptions, COMPILE_OPTIONS);
     this->LocalGenerator->AppendCompileOptions(flags, evaluatedOptions);
     *this->FlagFileStream << "# Custom options: " << relativeObj
@@ -792,7 +818,7 @@ void cmMakefileTargetGenerator::WriteObjectRuleFiles(
         this->GeneratorTarget->GetPchUseCompileOptions(config, lang);
     }
 
-    const std::string& evaluatedFlags =
+    std::string const& evaluatedFlags =
       genexInterpreter.Evaluate(pchOptions, COMPILE_OPTIONS);
 
     this->LocalGenerator->AppendCompileOptions(flags, evaluatedFlags);
@@ -804,9 +830,9 @@ void cmMakefileTargetGenerator::WriteObjectRuleFiles(
   // Add include directories from source file properties.
   std::vector<std::string> includes;
 
-  const std::string INCLUDE_DIRECTORIES("INCLUDE_DIRECTORIES");
+  std::string const INCLUDE_DIRECTORIES("INCLUDE_DIRECTORIES");
   if (cmValue cincludes = source.GetProperty(INCLUDE_DIRECTORIES)) {
-    const std::string& evaluatedIncludes =
+    std::string const& evaluatedIncludes =
       genexInterpreter.Evaluate(*cincludes, INCLUDE_DIRECTORIES);
     this->LocalGenerator->AppendIncludeDirectories(includes, evaluatedIncludes,
                                                    source);
@@ -820,9 +846,9 @@ void cmMakefileTargetGenerator::WriteObjectRuleFiles(
   std::set<std::string> defines;
 
   // Add source-specific preprocessor definitions.
-  const std::string COMPILE_DEFINITIONS("COMPILE_DEFINITIONS");
+  std::string const COMPILE_DEFINITIONS("COMPILE_DEFINITIONS");
   if (cmValue compile_defs = source.GetProperty(COMPILE_DEFINITIONS)) {
-    const std::string& evaluatedDefs =
+    std::string const& evaluatedDefs =
       genexInterpreter.Evaluate(*compile_defs, COMPILE_DEFINITIONS);
     this->LocalGenerator->AppendDefines(defines, evaluatedDefs);
     *this->FlagFileStream << "# Custom defines: " << relativeObj
@@ -832,7 +858,7 @@ void cmMakefileTargetGenerator::WriteObjectRuleFiles(
   std::string const defPropName =
     cmStrCat("COMPILE_DEFINITIONS_", configUpper);
   if (cmValue config_compile_defs = source.GetProperty(defPropName)) {
-    const std::string& evaluatedDefs =
+    std::string const& evaluatedDefs =
       genexInterpreter.Evaluate(*config_compile_defs, COMPILE_DEFINITIONS);
     this->LocalGenerator->AppendDefines(defines, evaluatedDefs);
     *this->FlagFileStream << "# Custom defines: " << relativeObj << "_DEFINES_"
@@ -905,6 +931,7 @@ void cmMakefileTargetGenerator::WriteObjectRuleFiles(
   vars.CMTargetName = this->GeneratorTarget->GetName().c_str();
   vars.CMTargetType =
     cmState::GetTargetTypeName(this->GeneratorTarget->GetType()).c_str();
+  vars.CMTargetLabels = this->GeneratorTarget->GetTargetLabelsString().c_str();
   vars.Language = lang.c_str();
   vars.Target = targetOutPathReal.c_str();
   vars.TargetPDB = targetOutPathPDB.c_str();
@@ -925,6 +952,7 @@ void cmMakefileTargetGenerator::WriteObjectRuleFiles(
   vars.ObjectFileDir = objectFileDir.c_str();
   vars.Flags = flags.c_str();
   vars.ISPCHeader = ispcHeaderForShell.c_str();
+  vars.Config = this->GetConfigName().c_str();
 
   std::string definesString = cmStrCat("$(", lang, "_DEFINES)");
 
@@ -976,7 +1004,7 @@ void cmMakefileTargetGenerator::WriteObjectRuleFiles(
     if (lang == "CUDA") {
       if (this->GeneratorTarget->GetPropertyAsBool(
             "CUDA_SEPARABLE_COMPILATION")) {
-        const std::string& rdcFlag =
+        std::string const& rdcFlag =
           this->Makefile->GetRequiredDefinition("_CMAKE_CUDA_RDC_FLAG");
         cudaCompileMode = cmStrCat(cudaCompileMode, rdcFlag, " ");
       }
@@ -989,7 +1017,7 @@ void cmMakefileTargetGenerator::WriteObjectRuleFiles(
         auto propName = cmStrCat("CUDA_", mode, "_COMPILATION");
         auto defName = cmStrCat("_CMAKE_CUDA_", mode, "_FLAG");
         if (this->GeneratorTarget->GetPropertyAsBool(propName)) {
-          const std::string& flag =
+          std::string const& flag =
             this->Makefile->GetRequiredDefinition(defName);
           cudaCompileMode = cmStrCat(cudaCompileMode, flag);
           useNormalCompileMode = false;
@@ -997,7 +1025,7 @@ void cmMakefileTargetGenerator::WriteObjectRuleFiles(
         }
       }
       if (useNormalCompileMode) {
-        const std::string& wholeFlag =
+        std::string const& wholeFlag =
           this->Makefile->GetRequiredDefinition("_CMAKE_CUDA_WHOLE_FLAG");
         cudaCompileMode = cmStrCat(cudaCompileMode, wholeFlag);
       }
@@ -1005,7 +1033,7 @@ void cmMakefileTargetGenerator::WriteObjectRuleFiles(
     }
 
     cmList compileCommands;
-    const std::string& compileRule = this->Makefile->GetRequiredDefinition(
+    std::string const& compileRule = this->Makefile->GetRequiredDefinition(
       "CMAKE_" + lang + "_COMPILE_OBJECT");
     compileCommands.assign(compileRule);
 
@@ -1105,13 +1133,13 @@ void cmMakefileTargetGenerator::WriteObjectRuleFiles(
       }
       vars.Flags = flagsWithDeps.c_str();
 
-      const auto& extraCommands = this->Makefile->GetSafeDefinition(
+      auto const& extraCommands = this->Makefile->GetSafeDefinition(
         cmStrCat("CMAKE_", lang, "_DEPENDS_EXTRA_COMMANDS"));
       if (!extraCommands.empty()) {
         compileCommands.append(extraCommands);
       }
 
-      const auto& depFormat = this->Makefile->GetRequiredDefinition(
+      auto const& depFormat = this->Makefile->GetRequiredDefinition(
         cmStrCat("CMAKE_", lang, "_DEPFILE_FORMAT"));
 
       if (depFormat == "msvc"_s) {
@@ -1124,7 +1152,7 @@ void cmMakefileTargetGenerator::WriteObjectRuleFiles(
                    this->LocalGenerator->ConvertToOutputFormat(
                      this->LocalGenerator->GetCurrentBinaryDirectory(),
                      cmOutputConverter::SHELL));
-        const auto& prefix = this->Makefile->GetSafeDefinition(
+        auto const& prefix = this->Makefile->GetSafeDefinition(
           cmStrCat("CMAKE_", lang, "_CL_SHOWINCLUDES_PREFIX"));
         depFilter += cmStrCat(" --filter-prefix=",
                               this->LocalGenerator->ConvertToOutputFormat(
@@ -1306,10 +1334,10 @@ void cmMakefileTargetGenerator::WriteTargetCleanRules()
 }
 
 bool cmMakefileTargetGenerator::WriteMakeRule(
-  std::ostream& os, const char* comment,
-  const std::vector<std::string>& outputs,
-  const std::vector<std::string>& depends,
-  const std::vector<std::string>& commands, bool in_help)
+  std::ostream& os, char const* comment,
+  std::vector<std::string> const& outputs,
+  std::vector<std::string> const& depends,
+  std::vector<std::string> const& commands, bool in_help)
 {
   bool symbolic = false;
   if (outputs.empty()) {
@@ -1472,11 +1500,11 @@ void cmMakefileTargetGenerator::WriteTargetDependRules()
     << this->GeneratorTarget->GetFortranModuleDirectory(working_dir)
     << "\")\n";
 
-  if (this->GeneratorTarget->IsFortranBuildingInstrinsicModules()) {
+  if (this->GeneratorTarget->IsFortranBuildingIntrinsicModules()) {
     *this->InfoFileStream
       << "\n"
       << "# Fortran compiler is building intrinsic modules.\n"
-      << "set(CMAKE_Fortran_TARGET_BUILDING_INSTRINSIC_MODULES ON) \n";
+      << "set(CMAKE_Fortran_TARGET_BUILDING_INTRINSIC_MODULES ON) \n";
   }
   /* clang-format on */
 
@@ -1533,8 +1561,7 @@ void cmMakefileTargetGenerator::WriteTargetDependRules()
               cmOutputConverter::SHELL)
          << " "
          << this->LocalGenerator->ConvertToOutputFormat(
-              cmSystemTools::CollapseFullPath(this->InfoFileNameFull),
-              cmOutputConverter::SHELL);
+              this->InfoFileNameFull, cmOutputConverter::SHELL);
   if (this->LocalGenerator->GetColorMakefile()) {
     depCmd << " \"--color=$(COLOR)\"";
   }
@@ -1569,7 +1596,7 @@ void cmMakefileTargetGenerator::WriteObjectDependRules(
 }
 
 void cmMakefileTargetGenerator::WriteDeviceLinkRule(
-  std::vector<std::string>& commands, const std::string& output)
+  std::vector<std::string>& commands, std::string const& output)
 {
   std::string architecturesStr =
     this->GeneratorTarget->GetSafeProperty("CUDA_ARCHITECTURES");
@@ -1586,7 +1613,7 @@ void cmMakefileTargetGenerator::WriteDeviceLinkRule(
   std::string const& relPath = localGen->GetHomeRelativeOutputPath();
 
   // Ensure there are no duplicates.
-  const std::vector<std::string> linkDeps = [&]() -> std::vector<std::string> {
+  std::vector<std::string> const linkDeps = [&]() -> std::vector<std::string> {
     std::vector<std::string> deps;
     this->AppendTargetDepends(deps, true);
     this->GeneratorTarget->GetLinkDepends(deps, this->GetConfigName(), "CUDA");
@@ -1601,8 +1628,8 @@ void cmMakefileTargetGenerator::WriteDeviceLinkRule(
     return deps;
   }();
 
-  const std::string objectDir = this->GeneratorTarget->ObjectDirectory;
-  const std::string relObjectDir =
+  std::string const objectDir = this->GeneratorTarget->ObjectDirectory;
+  std::string const relObjectDir =
     localGen->MaybeRelativeToCurBinDir(objectDir);
 
   // Construct a list of files associated with this executable that
@@ -1616,7 +1643,7 @@ void cmMakefileTargetGenerator::WriteDeviceLinkRule(
     cmStrCat(objectDir, "cmake_cuda_register.h");
 
   // Link device code for each architecture.
-  for (const std::string& architectureKind : architectures) {
+  for (std::string const& architectureKind : architectures) {
     std::string registerFileCmd;
 
     // The generated register file contains macros that when expanded
@@ -1632,9 +1659,9 @@ void cmMakefileTargetGenerator::WriteDeviceLinkRule(
     }
 
     // Clang always generates real code, so strip the specifier.
-    const std::string architecture =
+    std::string const architecture =
       architectureKind.substr(0, architectureKind.find('-'));
-    const std::string cubin =
+    std::string const cubin =
       cmStrCat(objectDir, "sm_", architecture, ".cubin");
 
     profiles += cmStrCat(" -im=profile=sm_", architecture, ",file=", cubin);
@@ -1650,14 +1677,14 @@ void cmMakefileTargetGenerator::WriteDeviceLinkRule(
   }
 
   // Combine all architectures into a single fatbinary.
-  const std::string fatbinaryCommand =
+  std::string const fatbinaryCommand =
     cmStrCat(this->Makefile->GetRequiredDefinition("CMAKE_CUDA_FATBINARY"),
              " -64 -cmdline=--compile-only -compress-all -link "
              "--embedded-fatbin=$@",
              profiles);
-  const std::string fatbinaryOutput =
+  std::string const fatbinaryOutput =
     cmStrCat(objectDir, "cmake_cuda_fatbin.h");
-  const std::string fatbinaryOutputRel =
+  std::string const fatbinaryOutputRel =
     cmStrCat(relPath, relObjectDir, "cmake_cuda_fatbin.h");
 
   localGen->WriteMakeRule(*this->BuildFileStream, nullptr, fatbinaryOutputRel,
@@ -1669,11 +1696,12 @@ void cmMakefileTargetGenerator::WriteDeviceLinkRule(
   vars.CMTargetName = this->GetGeneratorTarget()->GetName().c_str();
   vars.CMTargetType =
     cmState::GetTargetTypeName(this->GetGeneratorTarget()->GetType()).c_str();
-
+  vars.CMTargetLabels = this->GeneratorTarget->GetTargetLabelsString().c_str();
   vars.Language = "CUDA";
   vars.Object = output.c_str();
   vars.Fatbinary = fatbinaryOutput.c_str();
   vars.RegisterFile = registerFile.c_str();
+  vars.Config = this->GetConfigName().c_str();
 
   std::string linkFlags;
   this->GetDeviceLinkFlags(linkFlags, "CUDA");
@@ -1683,7 +1711,8 @@ void cmMakefileTargetGenerator::WriteDeviceLinkRule(
   vars.Flags = flags.c_str();
 
   std::string compileCmd = this->GetLinkRule("CMAKE_CUDA_DEVICE_LINK_COMPILE");
-  auto rulePlaceholderExpander = localGen->CreateRulePlaceholderExpander();
+  auto rulePlaceholderExpander = localGen->CreateRulePlaceholderExpander(
+    cmBuildStep::Link, this->GetGeneratorTarget(), "CUDA");
   rulePlaceholderExpander->ExpandRuleVariables(localGen, compileCmd, vars);
 
   commands.emplace_back(compileCmd);
@@ -1731,7 +1760,7 @@ void cmMakefileTargetGenerator::GenerateCustomRuleFile(
   }
 
   // Write the rule.
-  const std::vector<std::string>& outputs = ccg.GetOutputs();
+  std::vector<std::string> const& outputs = ccg.GetOutputs();
   bool const symbolic = this->WriteMakeRule(*this->BuildFileStream, nullptr,
                                             outputs, depends, commands);
 
@@ -1799,7 +1828,7 @@ void cmMakefileTargetGenerator::WriteObjectsVariable(
   *this->BuildFileStream << "# Object files for target "
                          << this->GeneratorTarget->GetName() << "\n"
                          << variableName << " =";
-  const auto& lineContinue = this->GlobalGenerator->LineContinueDirective;
+  auto const& lineContinue = this->GlobalGenerator->LineContinueDirective;
 
   cmValue pchExtension = this->Makefile->GetDefinition("CMAKE_PCH_EXTENSION");
 
@@ -1884,7 +1913,7 @@ private:
   std::string::size_type LengthLimit;
   std::string CurrentString;
   std::string NextObject;
-  const char* Space;
+  char const* Space;
 };
 
 void cmMakefileTargetGenerator::WriteObjectsStrings(
@@ -1914,7 +1943,7 @@ void cmMakefileTargetGenerator::WriteObjectsStrings(
 }
 
 void cmMakefileTargetGenerator::WriteTargetDriverRule(
-  const std::string& main_output, bool relink)
+  std::string const& main_output, bool relink)
 {
   // Compute the name of the driver target.
   std::string dir =
@@ -1927,7 +1956,7 @@ void cmMakefileTargetGenerator::WriteTargetDriverRule(
   // Build the list of target outputs to drive.
   std::vector<std::string> depends{ main_output };
 
-  const char* comment = nullptr;
+  char const* comment = nullptr;
   if (relink) {
     // Setup the comment for the preinstall driver.
     comment = "Rule to relink during preinstall.";
@@ -1960,7 +1989,7 @@ void cmMakefileTargetGenerator::AppendTargetDepends(
     return;
   }
 
-  const std::string& cfg = this->GetConfigName();
+  std::string const& cfg = this->GetConfigName();
 
   if (this->GeneratorTarget->HasLinkDependencyFile(cfg)) {
     depends.push_back(
@@ -1994,7 +2023,7 @@ void cmMakefileTargetGenerator::AppendObjectDepends(
 }
 
 void cmMakefileTargetGenerator::AppendLinkDepends(
-  std::vector<std::string>& depends, const std::string& linkLanguage)
+  std::vector<std::string>& depends, std::string const& linkLanguage)
 {
   this->AppendObjectDepends(depends);
 
@@ -2023,7 +2052,7 @@ void cmMakefileTargetGenerator::AppendLinkDepends(
 }
 
 std::string cmMakefileTargetGenerator::GetLinkRule(
-  const std::string& linkRuleVar)
+  std::string const& linkRuleVar)
 {
   std::string linkRule = this->Makefile->GetRequiredDefinition(linkRuleVar);
   if (this->GeneratorTarget->HasImplibGNUtoMS(this->GetConfigName())) {
@@ -2046,7 +2075,7 @@ void cmMakefileTargetGenerator::CloseFileStreams()
 }
 
 void cmMakefileTargetGenerator::CreateLinkScript(
-  const char* name, std::vector<std::string> const& link_commands,
+  char const* name, std::vector<std::string> const& link_commands,
   std::vector<std::string>& makefile_commands,
   std::vector<std::string>& makefile_depends)
 {
@@ -2082,7 +2111,7 @@ bool cmMakefileTargetGenerator::CheckUseResponseFileForObjects(
     "CMAKE_" + l + "_USE_RESPONSE_FILE_FOR_OBJECTS";
   if (cmValue val = this->Makefile->GetDefinition(responseVar)) {
     if (!val->empty()) {
-      return cmIsOn(val);
+      return val.IsOn();
     }
   }
 
@@ -2121,7 +2150,7 @@ bool cmMakefileTargetGenerator::CheckUseResponseFileForLibraries(
     "CMAKE_" + l + "_USE_RESPONSE_FILE_FOR_LIBRARIES";
   if (cmValue val = this->Makefile->GetDefinition(responseVar)) {
     if (!val->empty()) {
-      return cmIsOn(val);
+      return val.IsOn();
     }
   }
 
@@ -2130,7 +2159,7 @@ bool cmMakefileTargetGenerator::CheckUseResponseFileForLibraries(
 }
 
 std::string cmMakefileTargetGenerator::CreateResponseFile(
-  const std::string& name, std::string const& options,
+  std::string const& name, std::string const& options,
   std::vector<std::string>& makefile_depends, std::string const& language)
 {
   // FIXME: Find a better way to determine the response file encoding,
@@ -2179,13 +2208,14 @@ void cmMakefileTargetGenerator::CreateLinkLibs(
   bool useResponseFile, std::vector<std::string>& makefile_depends,
   std::string const& linkLanguage, ResponseFlagFor responseMode)
 {
-  std::string frameworkPath;
-  std::string linkPath;
-  cmComputeLinkInformation* pcli =
-    this->GeneratorTarget->GetLinkInformation(this->GetConfigName());
-  this->LocalGenerator->OutputLinkLibraries(pcli, linkLineComputer, linkLibs,
-                                            frameworkPath, linkPath);
-  linkLibs = frameworkPath + linkPath + linkLibs;
+  if (cmComputeLinkInformation* pcli =
+        this->GeneratorTarget->GetLinkInformation(this->GetConfigName())) {
+    std::string frameworkPath;
+    std::string linkPath;
+    this->LocalGenerator->OutputLinkLibraries(pcli, linkLineComputer, linkLibs,
+                                              frameworkPath, linkPath);
+    linkLibs = frameworkPath + linkPath + linkLibs;
+  }
 
   if (useResponseFile &&
       linkLibs.find_first_not_of(' ') != std::string::npos) {
@@ -2230,26 +2260,22 @@ void cmMakefileTargetGenerator::CreateObjectLists(
     std::string const responseFlag = this->GetResponseFlag(responseMode);
 
     // Write a response file for each string.
-    const char* sep = "";
+    char const* sep = "";
     for (unsigned int i = 0; i < object_strings.size(); ++i) {
       // Number the response files.
-      std::string responseFileName =
-        (responseMode == Link) ? "objects" : "deviceObjects";
-      responseFileName += std::to_string(i + 1);
-      responseFileName += ".rsp";
+      std::string responseFileName = cmStrCat(
+        (responseMode == Link) ? "objects" : "deviceObjects", i + 1, ".rsp");
 
       // Create this response file.
       std::string objects_rsp = this->CreateResponseFile(
         responseFileName, object_strings[i], makefile_depends, linkLanguage);
 
-      // Separate from previous response file references.
-      buildObjs += sep;
+      buildObjs +=
+        cmStrCat(sep, // Separate from previous response file references.
+                 responseFlag, // Reference the response file.
+                 this->LocalGenerator->ConvertToOutputFormat(
+                   objects_rsp, cmOutputConverter::SHELL));
       sep = " ";
-
-      // Reference the response file.
-      buildObjs += responseFlag;
-      buildObjs += this->LocalGenerator->ConvertToOutputFormat(
-        objects_rsp, cmOutputConverter::SHELL);
     }
   } else if (useLinkScript) {
     if (!useArchiveRules) {
@@ -2264,8 +2290,8 @@ void cmMakefileTargetGenerator::CreateObjectLists(
 }
 
 void cmMakefileTargetGenerator::AddIncludeFlags(std::string& flags,
-                                                const std::string& lang,
-                                                const std::string& /*config*/)
+                                                std::string const& lang,
+                                                std::string const& /*config*/)
 {
   std::string const responseVar =
     cmStrCat("CMAKE_", lang, "_USE_RESPONSE_FILE_FOR_INCLUDES");
